DIRS_TO_IGNORE = [
    '__pycache__',
    '.git',
    '.pytest_cache',
    'build',
    'dist',
    'htmlcov',
    'node_modules',
    'vendor',
]

FILES_TO_IGNORE = [
    '.bmp'
    '.eot',
    '.gif',
    '.ico',
    '.jpeg',
    '.jpg',
    '.pdf',
    '.png',
    '.ttf',
    '.woff',
    '.zip',
    '',
]
